'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Dependencies
var debug = require('debug')('node-cba-netbank');
var moment = require('./moment');
var WebClient = require('./web');
var parser = require('./parser');
var Url = require('url');
var uniq = require('lodash/uniq');

// Constant
var NETBANK_HOST = 'https://www.my.commbank.com.au';
var PATH = {
  LOGON: '/netbank/Logon/Logon.aspx',
  HOME: '/netbank/Portfolio/Home/Home.aspx?RID=:RID&SID=:SID'
};

//  Utilities
var concat = function concat(a, b) {
  return uniq(a.concat(b));
};

//  API

var API = function () {
  function API(credentials) {
    _classCallCheck(this, API);

    this.credentials = credentials;
    this.web = new WebClient();
    this.host = NETBANK_HOST;
  }

  _createClass(API, [{
    key: 'logon',
    value: function logon() {
      var _this = this;

      //  retrieve the logon page
      return this.web.get(this.getUrl(PATH.LOGON)
      //  Parse the page to get logon form
      ).then(function (resp) {
        return _this.refreshBase(resp);
      }).then(parser.parseForm).then(function (resp) {
        return (
          //  fill the logon form and submit
          _this.web.post({
            url: _this.getUrl(PATH.LOGON),
            form: Object.assign({}, resp.form, {
              txtMyClientNumber$field: _this.credentials.username,
              txtMyPassword$field: _this.credentials.password,
              chkRemember$field: 'on',
              //  and make JS detector happy
              JS: 'E'
            })
          })
        );
      }).then(function (resp) {
        return _this.refreshBase(resp);
      }
      //  parse the home page to retrieve the accounts list
      ).then(parser.parseHomePage);
    }

    //  Get transaction history data for given time period.
    //
    //  * `from`: Default download begin at 6 years ago. FYI, I tried 8 years
    //    without getting any error message, however, keep in mind that bank
    //    usually only stored 2 years transactions history data.
    //  * `to`: Default value is today.

  }, {
    key: 'getTransactionHistory',
    value: function getTransactionHistory(account) {
      var _this2 = this;

      var from = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : moment().subtract(6, 'years').format(moment.formats.default);
      var to = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : moment().format(moment.formats.default);

      debug(`getTransactionHistory(account: ${account.name} [${account.number}] => ${account.available})`);
      //  retrieve post form and key for the given account
      return this.web.get(this.getUrl(account.link)).then(parser.parseTransactionPage).then(function (resp) {
        return _this2.refreshBase(resp);
      }).then(function (resp) {
        var acc = Object.assign({}, account, { link: Url.parse(resp.url).path });
        if (resp.accounts !== null) {
          acc.key = resp.accounts.find(function (a) {
            return a.number === account.number;
          }).key;
        }

        //  if the transaction section is lazy loading, we need do a panel update
        //  first, before the real search.
        if (!resp.form.ctl00$BodyPlaceHolder$radioSwitchSearchType$field$) {
          return _this2.lazyLoading(resp, acc).then(function (r) {
            return _this2.getTransactionsByDate(r, acc, from, to);
          } //  attach pendings
          ).then(function (r) {
            return Object.assign({}, r, { pendings: resp.pendings });
          });
        }
        return _this2.getTransactionsByDate(resp, acc, from, to
        //  attach pending
        ).then(function (r) {
          return Object.assign({}, r, { pendings: resp.pendings });
        }).then(function (r) {
          debug(`getTransactionHistory(): Total received ${r.transactions.length} transactions.`);
          return r;
        });
      });
    }

    //  Web API

    //  If the search panel is lazy loading, not only the transactions data is not in
    //  the page, the search page widget is not ready as well. So, search request will
    //  fail. To workaround such problem, we send an update panel partial callback
    //  first,let server side prepared the search panel and download transaction.
    //  Then, do the real search using the information from the parital callback result.

  }, {
    key: 'lazyLoading',
    value: function lazyLoading(response, account) {
      debug(`lazyLoading(account: ${account.name} [${account.number}] => ${account.available}))`);
      return this.web.post({
        url: this.getUrl(account.link),
        form: Object.assign({}, response.form, {
          //  Send partial request
          ctl00$ctl00: 'ctl00$BodyPlaceHolder$UpdatePanelForAjax|ctl00$BodyPlaceHolder$UpdatePanelForAjaxh',
          __EVENTTARGET: 'ctl00$BodyPlaceHolder$UpdatePanelForAjax',
          __EVENTARGUMENT: 'doPostBackApiCall|LoadRecentTransactions|false'
        }),
        partial: true
      }).then(parser.parseViewState).then(function (resp) {
        return parser.parseForm(resp).catch(function () {
          return resp;
        });
      }).then(function (resp) {
        return Object.assign({}, resp, { form: Object.assign({}, response.form, resp.form) });
      });
    }
  }, {
    key: 'getMoreTransactions',
    value: function getMoreTransactions(response, account) {
      var _this3 = this;

      debug(`getMoreTransactions(account: ${account.name} [${account.number}] => ${account.available})`);
      var form = Object.assign({}, response.form, {
        // fill the form
        ctl00$ctl00: 'ctl00$BodyPlaceHolder$UpdatePanelForAjax|ctl00$BodyPlaceHolder$UpdatePanelForAjax',
        __EVENTTARGET: 'ctl00$BodyPlaceHolder$UpdatePanelForAjax',
        __EVENTARGUMENT: 'doPostBackApiCall|LoadTransactions|{"ClearCache":"false","IsSorted":false,"IsAdvancedSearch":true,"IsMonthSearch":false}'
      });
      // send the request
      return this.web.post({
        url: this.getUrl(account.link),
        form,
        partial: true
      }).then(parser.parseTransactions).then(function (resp) {
        return _this3.refreshBase(resp);
      }).then(function (resp) {
        if (!resp.more || resp.limit) {
          //  There is no more transactions or reached the limit.
          return resp;
        }
        return _this3.getMoreTransactions(Object.assign({}, resp, { form }), Object.assign({}, account, { link: Url.parse(resp.url).path })
        //  concat more transactions.
        ).then(function (r) {
          return Object.assign({}, r, { form, transactions: concat(resp.transactions, r.transactions) });
        }
        //  Ignore the error as we have got some transactions already
        ).catch(function (error) {
          debug(error);
          return resp;
        });
      });
    }
  }, {
    key: 'getTransactionsByDate',
    value: function getTransactionsByDate(response, account, from, to) {
      var _this4 = this;

      debug(`getTransactionsByDate(account: ${account.name} [${account.number}] => ${account.available}, from: ${from}, to: ${to})`);
      var form = Object.assign({}, response.form, {
        // fill the form
        ctl00$ctl00: 'ctl00$BodyPlaceHolder$updatePanelSearch|ctl00$BodyPlaceHolder$lbSearch',
        __EVENTTARGET: 'ctl00$BodyPlaceHolder$lbSearch',
        __EVENTARGUMENT: '',
        ctl00$BodyPlaceHolder$searchTypeField: '1',
        ctl00$BodyPlaceHolder$radioSwitchDateRange$field$: 'ChooseDates',
        ctl00$BodyPlaceHolder$dateRangeField: 'ChooseDates',
        ctl00$BodyPlaceHolder$fromCalTxtBox$field: from,
        ctl00$BodyPlaceHolder$toCalTxtBox$field: to,
        //  Add this for partial update
        ctl00$BodyPlaceHolder$radioSwitchSearchType$field$: 'AllTransactions'
      });

      return this.web.post({
        url: this.getUrl(account.link),
        form,
        partial: true
      }).then(parser.parseTransactions).then(function (resp) {
        return _this4.refreshBase(resp);
      }).then(function (resp) {
        if (!resp.more || resp.limit) {
          //  there is no more transactions or reached the limit.
          return resp;
        }
        //  There are more transactions available.
        return _this4.getMoreTransactions(Object.assign({}, resp, { form }), Object.assign({}, account, { link: Url.parse(resp.url).path })
        //  concat more transactions.
        ).then(function (r) {
          return Object.assign({}, r, { form, transactions: concat(resp.transactions, r.transactions) });
        }).then(function (r) {
          debug(`getTransactionsByDate(): getMoreTransactions(): More => ${r.more}, Limit => ${r.limit}`);
          if (r.more && r.limit) {
            //  if there are more transactions, however it reaches limit
            //  we need to send another search request to overcome the limit.
            throw Object.assign(new Error('Reach transations limit'), { response: r });
          }
          return r;
        }).catch(function (error) {
          //  an error happend during load more, it means that it may have more,
          //  however, some restriction made it stopped, so we call it again,
          //  but this time, we use the eariliest date from the transactions
          //  we retrieved so far as the toDate, so it might workaround this
          //  problem.

          debug(error);

          //  if there is a `response` object attached to `error` object, that means
          //  it's just reach the limit, and contains transations. Otherwise, it don't
          //  have the transactions, a real error, then use previous `resp` instead.
          var r = error.response || resp;
          //  find the earliest date as new 'to' date.
          var timestamp = r.transactions[0].timestamp;
          r.transactions.forEach(function (t) {
            if (timestamp > t.timestamp) {
              timestamp = t.timestamp;
            }
          });
          var newTo = moment(timestamp).format(moment.formats.default);

          // Call self again
          debug(`Call getTransactionsByDate() again with new 'to' date (${to} => ${newTo})`);
          return _this4.getTransactionsByDate(Object.assign({}, r, { form }), Object.assign({}, account, { link: Url.parse(r.url).path }), from, newTo
          //  concat more transactions
          ).then(function (rr) {
            return Object.assign({}, rr, { form, transactions: concat(r.transactions, rr.transactions) });
          }).catch(function (err) {
            //  cannot call it again, but we got some transactions at least,
            //  so, just call it a success.
            debug(err);
            debug('getTransactionsByDate(): failed to call self again to load more');
            return Object.assign({}, r, { form, transactions: r.transactions });
          });
        });
      });
    }

    //  Utilities
    //  Return `${this.base}+${path}`

  }, {
    key: 'getUrl',
    value: function getUrl(path) {
      return Url.resolve(this.host, path);
    }

    //  Change the BASE link if it's been redirected.

  }, {
    key: 'refreshBase',
    value: function refreshBase(resp) {
      var oldLink = Url.parse(this.host);
      var newLink = Url.parse(resp.url);

      if (oldLink.host !== newLink.host) {
        debug(`refreshBase(${oldLink.host} => ${newLink.host}`);
        oldLink.host = newLink.host;
        this.host = Url.format(oldLink);
      }
      return resp;
    }
  }]);

  return API;
}();

//  Exports


module.exports = API;