'use strict';

//  Dependencies
var chalk = require('chalk');
var Table = require('easy-table');
var moment = require('./moment');

//  Functions
var currency = function currency(amount) {
  return amount >= 0 ? chalk.green.bold(`$${amount.toFixed(2)}`) : chalk.red.bold(`$${amount.toFixed(2)}`);
};
var account = function account(a) {
  return `${chalk.bold(a.name)} \t(${chalk.red(a.bsb)} ${chalk.red(a.account)})` + `\t Balance: ${currency(a.balance)} ` + `\t Available Funds: ${currency(a.available)}`;
};
var accounts = function accounts(accs) {
  return Table.print(accs.map(function (a) {
    return {
      Name: chalk.bold(a.name),
      Number: `${chalk.red(a.bsb)} ${chalk.red(a.account)}`,
      Balance: `${currency(a.balance)}`,
      Available: `${currency(a.available)}`
    };
  }));
};
var transactions = function transactions(ts) {
  return Table.print(ts.map(function (t) {
    return {
      Time: chalk.italic.cyan(moment(t.timestamp).format('YYYY-MM-DD HH:mm')),
      Description: t.pending ? chalk.gray(t.description) : chalk.white(t.description),
      Amount: currency(t.amount),
      Balance: t.pending ? '' : currency(t.balance)
    };
  }));
};

module.exports = {
  currency,
  account,
  accounts,
  transactions
};