'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Dependencies
var moment = require('./moment');
var API = require('./api');
var Render = require('./render');

var chalk = require('chalk');
var debug = require('debug')('node-cba-netbank');
var inquirer = require('inquirer');

//  Constant
var msgQuit = 'quit';
var tagQuit = chalk.red('<Quit>');

/* eslint-disable class-methods-use-this */

var UI = function () {
  function UI() {
    _classCallCheck(this, UI);

    this.accounts = [];
  }

  _createClass(UI, [{
    key: 'start',
    value: function start(argv) {
      var _this = this;

      return new Promise(function (resolve) {
        //  check given credential
        var questions = [];

        //  client number
        if (!argv.username) {
          questions.push({
            type: 'input',
            name: 'username',
            message: 'Client number:',
            validate: function validate(input) {
              return (/^\d{1,8}$/.test(input)
              );
            }
          });
        }

        //  password
        if (!argv.password) {
          questions.push({
            type: 'password',
            name: 'password',
            message: 'Password:',
            validate: function validate(input) {
              return (/^[\w\d-]{4,16}$/.test(input)
              );
            }
          });
        }

        return resolve(questions.length > 0 ? inquirer.prompt(questions) : argv);
      }).then(function (c) {
        return _this.logon(c);
      }).then(function () {
        return _this.chooseAccountAndShowHistory(argv.months);
      }).catch(function (error) {
        debug(error);
        console.log('Bye!');
      });
    }
  }, {
    key: 'logon',
    value: function logon(credential) {
      var _this2 = this;

      this.api = new API(credential);
      console.log(`Logon as account ${credential.username} ...`);
      return this.api.logon(credential).catch(function () {
        return inquirer.prompt({
          type: 'confirm',
          name: 'tryagain',
          message: 'Failed to logged in, try again?'
        }).then(function (answer) {
          if (answer.tryagain) {
            console.log();
            //  retry without give username/password, so user can give a new one to try.
            return _this2.start(Object.assign({}, credential, { username: '', password: '' }));
          }
          throw new Error(msgQuit);
        });
      }).then(function (resp) {
        _this2.accounts = resp.accounts;
        return _this2.accounts;
      });
    }
  }, {
    key: 'chooseAccountAndShowHistory',
    value: function chooseAccountAndShowHistory(months) {
      var _this3 = this;

      return this.selectAccount().then(function (account) {
        return _this3.downloadHistoryAndShow(account, moment().subtract(months, 'months').format(moment.formats.default));
      }).then(function () {
        return _this3.chooseAccountAndShowHistory(months);
      });
    }
  }, {
    key: 'selectAccount',
    value: function selectAccount() {
      var _this4 = this;

      return inquirer.prompt({
        type: 'list',
        name: 'account',
        message: 'Which account?',
        choices: this.accounts.map(function (a) {
          return { name: Render.account(a), value: a };
        }).concat([tagQuit])
      }).then(function (answer) {
        if (answer.account === tagQuit) {
          throw new Error(msgQuit);
        }
        var account = _this4.accounts.find(function (v) {
          return answer.account.number === v.number;
        });
        if (!account) {
          throw new Error(msgQuit);
        }
        debug(`selectAccount(${Render.account(answer.account)}): found account ${Render.account(account)}`);
        return account;
      });
    }
  }, {
    key: 'downloadHistory',
    value: function downloadHistory(account, from, to) {
      console.log(`Downloading history [${from} => ${to}] ...`);
      return this.api.getTransactionHistory(account, from, to);
    }
  }, {
    key: 'downloadHistoryAndShow',
    value: function downloadHistoryAndShow(account, from) {
      var to = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : moment().format(moment.formats.default);

      return this.downloadHistory(account, from, to).then(function (history) {
        var allTransactions = history.pendings.map(function (t) {
          return Object.assign({}, t, { pending: true });
        }).concat(history.transactions);
        console.log(Render.transactions(allTransactions));
        console.log(`Total ${history.transactions.length} transactions and ${history.pendings.length} pending transactions.`);
        return history;
      });
    }
  }]);

  return UI;
}();

module.exports = UI;