'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Dependencies
var debug = require('debug')('node-cba-netbank');
var fs = require('mz/fs');
var isString = require('lodash/isString');
var request = require('request-promise');
var truncate = require('lodash/truncate');

// Initialisation
var DEFAULT_OPTION = {
  jar: true,
  followAllRedirects: true,
  resolveWithFullResponse: true,
  headers: {
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36',
    'Cache-Control': 'no-cache'
  }
};

// Utilities
function shorten(form) {
  var shortenForm = {};
  Object.keys(form).forEach(function (key) {
    shortenForm[key] = truncate(form[key], { length: 200 });
  });
  return shortenForm;
}

function doRequest(req) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var sequence = Date.now();
  var partial = params.partial;

  var headers = Object.assign({}, params.headers);
  if (partial) {
    headers['X-MicrosoftAjax'] = 'Delta=true';
  }
  var myParams = Object.assign({ method: 'GET' }, params, { headers });

  if (debug.enabled) {
    debug(`${myParams.method} ${myParams.url.substring(0, 80)}...`);
    debug(`headers => ${JSON.stringify(shorten(myParams.headers))}`);
    if (myParams.method === 'POST') {
      debug(`form => ${JSON.stringify(shorten(myParams.form))}`);
    }
    fs.writeFile(`log/${sequence}-1-request.json`, JSON.stringify(shorten(myParams), null, 2));
  }

  return req(myParams).then(function (response) {
    if (debug.enabled) {
      fs.writeFile(`log/${sequence}-2-response.json`, JSON.stringify({
        request: response.request,
        status: {
          code: response.statusCode,
          message: response.statusMessage
        },
        headers: response.headers,
        body: `${response.body.substring(0, 1000)}...`
      }, null, 2));
      fs.writeFile(`log/${sequence}-3-response-body.html`, response.body);
    }
    return { url: response.request.href, headers: response.headers, body: response.body };
  });
}

var WebClient = function () {
  function WebClient() {
    _classCallCheck(this, WebClient);

    this.request = request.defaults(DEFAULT_OPTION);
  }

  _createClass(WebClient, [{
    key: 'get',
    value: function get(params) {
      return isString(params) ? doRequest(this.request, { url: params }) : doRequest(this.request, params);
    }
  }, {
    key: 'post',
    value: function post(params) {
      return doRequest(this.request, Object.assign({ method: 'POST' }, params));
    }
  }]);

  return WebClient;
}();

//  Export


module.exports = WebClient;